package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.activiti.service.ActivitiService;
import com.qmth.sop.business.bean.params.FlowApproveParam;
import com.qmth.sop.business.bean.result.DingApplyDoneResult;
import com.qmth.sop.business.bean.result.DingApplyUnDoneResult;
import com.qmth.sop.business.entity.SysUser;
import com.qmth.sop.business.entity.TBDing;
import com.qmth.sop.business.entity.TBDingApply;
import com.qmth.sop.business.entity.TFCustomFlowEntity;
import com.qmth.sop.business.mapper.TBDingApplyMapper;
import com.qmth.sop.business.service.BasicAttachmentService;
import com.qmth.sop.business.service.TBDingApplyService;
import com.qmth.sop.business.service.TBDingService;
import com.qmth.sop.business.service.TFCustomFlowEntityService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.DingExceptionApproveEnum;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import com.qmth.sop.common.enums.FlowApprovePassEnum;
import com.qmth.sop.common.util.ServletUtil;
import org.activiti.engine.TaskService;
import org.activiti.engine.task.Task;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>
 * 考勤补卡申请表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
@Service
public class TBDingApplyServiceImpl extends ServiceImpl<TBDingApplyMapper, TBDingApply> implements TBDingApplyService {

    @Resource
    TFCustomFlowEntityService tfCustomFlowEntityService;

    @Resource
    ActivitiService activitiService;

    @Resource
    TBDingService tbDingService;

    @Resource
    BasicAttachmentService basicAttachmentService;

    @Resource
    TaskService taskService;

    /**
     * 新增考勤异常补卡申请
     *
     * @param tbDingApply
     * @return
     * @throws InterruptedException
     */
    @Override
    @Transactional
    public Boolean saveDingApply(TBDingApply tbDingApply) throws InterruptedException {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        TBDing tbDing = tbDingService.getById(tbDingApply.getDingId());
        Optional.ofNullable(tbDing).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("考勤记录为空"));

        Map<String, Object> map = activitiService.taskApprove(new FlowApproveParam(tbDingApply.getFlowDeploymentId(), FlowApprovePassEnum.START, !CollectionUtils.isEmpty(tbDingApply.getApproveUserIds()) ? tbDingApply.getApproveUserIds() : Arrays.asList(String.valueOf(sysUser.getId())), tbDing.getCrmNo()));
        TFCustomFlowEntity tfCustomFlowEntity = (TFCustomFlowEntity) map.get(SystemConstant.FLOW_ENTITY);
        tbDingApply.setCode(tfCustomFlowEntity.getCode(), tbDingApply.getAttachmentIdList(), sysUser.getId());
        tfCustomFlowEntity.setObjId(tbDingApply.getId());
        tfCustomFlowEntityService.updateById(tfCustomFlowEntity);
        return this.saveOrUpdate(tbDingApply);
    }

    /**
     * 异常待审核接口
     *
     * @param iPage
     * @param serviceId
     * @param name
     * @param supplierId
     * @param customName
     * @param startTime
     * @param endTime
     * @param applyStartTime
     * @param applyEndTime
     * @return
     */
    @Override
    public IPage<DingApplyUnDoneResult> flowTaskUnDoneList(IPage<Map> iPage, Long serviceId, String name, Long supplierId, String customName, String startTime, String endTime, Long applyStartTime, Long applyEndTime) throws Exception {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        IPage<DingApplyUnDoneResult> dingApplyUnDoneResultIPage = this.baseMapper.flowTaskUnDoneList(iPage, Arrays.asList(sysUser.getId().toString()), serviceId, name, supplierId, customName, startTime, endTime, applyStartTime, applyEndTime);
        if (Objects.nonNull(dingApplyUnDoneResultIPage) && !CollectionUtils.isEmpty(dingApplyUnDoneResultIPage.getRecords())) {
            for (DingApplyUnDoneResult d : dingApplyUnDoneResultIPage.getRecords()) {
                if (Objects.nonNull(d.getAttachmentPaths())) {
                    String[] strs = d.getAttachmentPaths().split(";");
                    StringJoiner stringJoiner = new StringJoiner(";");
                    for (int i = 0; i < strs.length; i++) {
                        String path = basicAttachmentService.filePreview(strs[i]);
                        stringJoiner.add(path);
                    }
                    d.setAttachmentPaths(stringJoiner.toString());
                }
            }
        }
        return dingApplyUnDoneResultIPage;
    }

    /**
     * 异常已审核接口
     *
     * @param iPage
     * @param serviceId
     * @param name
     * @param supplierId
     * @param dingExceptionApprove
     * @param customName
     * @param startTime
     * @param endTime
     * @param applyStartTime
     * @param applyEndTime
     * @return
     */
    @Override
    public IPage<DingApplyDoneResult> flowTaskDoneList(IPage<Map> iPage, Long serviceId, String name, Long supplierId, DingExceptionApproveEnum dingExceptionApprove, String customName, String startTime, String endTime, Long applyStartTime, Long applyEndTime) throws Exception {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        IPage<DingApplyDoneResult> dingApplyDoneResultIPage = this.baseMapper.flowTaskDoneList(iPage, Arrays.asList(sysUser.getId().toString()), serviceId, name, supplierId, Objects.nonNull(dingExceptionApprove) ? dingExceptionApprove.getTitle() : null, customName, startTime, endTime, applyStartTime, applyEndTime);
        if (Objects.nonNull(dingApplyDoneResultIPage) && !CollectionUtils.isEmpty(dingApplyDoneResultIPage.getRecords())) {
            for (DingApplyDoneResult d : dingApplyDoneResultIPage.getRecords()) {
                if (Objects.nonNull(d.getAttachmentPaths())) {
                    String[] strs = d.getAttachmentPaths().split(";");
                    StringJoiner stringJoiner = new StringJoiner(";");
                    for (int i = 0; i < strs.length; i++) {
                        String path = basicAttachmentService.filePreview(strs[i]);
                        stringJoiner.add(path);
                    }
                    d.setAttachmentPaths(stringJoiner.toString());
                }
            }
        }
        return dingApplyDoneResultIPage;
    }

    /**
     * 考勤异常审核接口
     *
     * @param taskId
     * @param dingExceptionApprove
     * @param userId
     * @return
     */
    @Override
    public Boolean flowApprove(Long taskId, DingExceptionApproveEnum dingExceptionApprove, Long userId) throws InterruptedException {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        userId = Objects.nonNull(sysUser) ? sysUser.getId() : userId;
        Task task = taskService.createTaskQuery().taskId(String.valueOf(taskId)).singleResult();
        Optional.ofNullable(task).orElseThrow(() -> ExceptionResultEnum.FLOW_TASK_NO_DATA.exception());

        TFCustomFlowEntity tfCustomFlowEntity = tfCustomFlowEntityService.getOne(new QueryWrapper<TFCustomFlowEntity>().lambda().eq(TFCustomFlowEntity::getFlowId, Long.parseLong(task.getProcessInstanceId())));
        activitiService.taskApprove(new FlowApproveParam(taskId, FlowApprovePassEnum.PASS, Arrays.asList(String.valueOf(sysUser.getId())), tfCustomFlowEntity.getCrmNo(), dingExceptionApprove.getTitle()));
        return true;
    }
}
